import pandas as pd
import numpy as np
import os
from pathlib import Path
from enum import Enum
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

class parameters(Enum):
    pass

class validation():
    def init(self):
        pass

    def readin_country_data(self):
        path = os.path.dirname(Path(__file__).resolve().parents[0])
        country_data = pd.read_csv(f'{path}\\Input\\Additional_Information\\country_info.csv', encoding = "ISO-8859-1")
        return country_data
    
    def readin_external_data(self):
        path = os.path.dirname(Path(__file__).resolve().parents[0])
        external_data = pd.read_csv(f'{path}\\Input\\Additional_Information\\external_model_data.csv', encoding = "ISO-8859-1")
        return external_data

    def model_difference(self, data: pd.DataFrame):
        data_gfpm = data[data.ID==0].reset_index(drop=True)
        data_gfpm = data_gfpm[['RegionCode','domain',"CommodityCode","quantity"]]
        for i in data.ID.unique():
            sc_name = list(data.Scenario[data.ID==i])[0]
            data_gfpmpt = data[data.ID==i].reset_index(drop=True)
            data_gfpm[sc_name] = data_gfpm.quantity - data_gfpmpt.quantity
        return data_gfpm
    
    def model_corrcoef(self, data: pd.DataFrame, unit: str = 'quantity'):
        data_gfpm = data[data.ID==0].reset_index(drop=True)
        column_list = ['Continent','domain','CommodityCode','year']
        data_gfpm = data_gfpm[column_list + [unit]]
        cor_df = pd.DataFrame()
        for column in column_list:
            corr_full = pd.DataFrame()
            for i in data.ID.unique():
                sc_name = list(data.Scenario[data.ID==i])[0]
                data_gfpmpt = data[data.ID==i].reset_index(drop=True)
                data_gfpm[sc_name] = data_gfpmpt.quantity
                correlations = data_gfpm.groupby(column)[[unit,sc_name]].corr().iloc[0::2,-1].reset_index()
                corr_full = pd.concat([corr_full,correlations[sc_name]], axis=1)
            correlations.rename(columns={column: "ID"}, inplace=True)
            correlations = pd.concat([correlations["ID"], corr_full], axis=1)
            correlations["column"] = column
            cor_df = pd.concat([cor_df,correlations],axis=0)

        color_palette=['royalblue', 'peru', 'forestgreen', 
               'orangered', 'darkviolet', 'darkcyan', 
               'brown', 'pink', 'olive', 'grey']
        width = 0.5  # the width of the bars
        data_barplot = cor_df[cor_df.column == "domain"]
        
        num_sc = len(data_barplot.ID)
        name_df = pd.DataFrame()
        cor_df = data_barplot[data_barplot.columns[1:-2]]
        for i in range(0,num_sc):
            title_name = data_barplot[data_barplot.columns[0:1]].iloc[i:i+1]
            name_df = pd.concat([name_df,title_name])
        
        fig, axes =  plt.subplots(5, 1, sharex=False, sharey= False, figsize=(3,9)) 
        for i in range(0,num_sc):
            value = pd.DataFrame(cor_df.iloc[i])
            mean_val = value.mean()
            diff_val = value - mean_val
            ylim_max = diff_val.max().values * 1.05
            ylim_min = diff_val.min().values * 0.95
            diff_val.plot.bar(grid=True, color=color_palette[i], width=0.9, ax=axes[i])
            axes[i].title.set_text(str(name_df.iloc[i].values))
            axes[i].get_legend().remove()
            axes[i].set_ylim([ylim_min,ylim_max])
            if i < 4:
                axes[i].get_xaxis().set_visible(False)
        
        fig.tight_layout(pad=1.0)
        #plt.show()


            # courses = data_plot.columns
            # values = data_plot.iloc[i].values
            # fig = plt.figure(figsize = (10, 5))
            # values.plt.bar(courses,axs = axs[i])#, color='royalblue',
                          #color_paette, 
                          #width = width)
            # axs[i].plt.xticks(rotation=30, ha='right') 
            # axs[i].plt.ylim([min(values)*0.995, max(values)*1.005])   
            # axs[i].plt.xlabel("")
            # axs[i].plt.ylabel("")
            # axs[i].plt.title(title_name.values[0][0])
            # plt.show()
        return cor_df
    
    def validation(self, data: pd.DataFrame):
        data_vali = data[data.ID==0].reset_index(drop=True)
        data_std_prev = []
        for i in data.ID.unique():
            print(data.Scenario[data.ID == i][0])
            sc_name = data.Scenario[data.ID == i][0]
            data_gfpm = data_vali
            data_gfpmpt = data[data.ID==i].reset_index(drop=True)
            data_gfpm["vali"] = data_gfpm.quantity - data_gfpmpt.quantity
            data_std = pd.DataFrame(data_gfpm.groupby(['RegionCode','domain',"CommodityCode"])["vali"].std()).reset_index()
            data_std.columns = ["Region", "Domain","Commodity",sc_name]
            data_std_index = data_std[["Region", "Domain","Commodity"]]
            data_std_prev = pd.concat([pd.DataFrame(data_std_prev),pd.DataFrame(data_std[sc_name])], axis=1)
        data_std = pd.concat([data_std_index, data_std_prev], axis=1)
        fourth_quantile = data_std[data_std.columns[3]].quantile([.75])
        data_std = data_std.sort_values(by=data_std.columns[3], ascending=False)
        #data_std_count = pd.DataFrame(data_std.groupby(['Region','Domain']).count()).reset_index()
        return data_std
    
    def filter_data(self, data: pd.DataFrame, country_data: pd.DataFrame):
        
        data_filtered = pd.DataFrame()

        for data_key in data.keys():
            data_fin = pd.DataFrame() 
            #  filter carbon
            if data_key == 'Carbon':
                data_temp = data[data_key]
                data_info = data_temp[['RegionCode', 'Period', 'Scenario', 'Model']]

                col_list = ['CarbonStockBiomass [MtCO2]', 'CarbonStockHWP [MtCO2]']  
                for col in col_list:
                    data_reformated = pd.DataFrame()
                    data_col = data_temp[col]
                    data_reformated['Data'] = data_col
                    data_reformated['Parameter'] = col
                    data_reformated = pd.concat([data_info, data_reformated], axis=1)
                    data_fin = pd.concat([data_fin, data_reformated], axis=0).reset_index(drop=True)

            # filter weighted roundwood prices and roundwood harvest
            if data_key == 'Data':
                # compute weighted roundwood prices
                data_temp = data[data_key]
                data_temp = data_temp[[x in [78, 81] for x in data_temp['CommodityCode']]].reset_index(drop=True)
                data_temp = data_temp[data_temp['domain'] == 'Supply'].reset_index(drop=True)
                data_temp = data_temp[data_temp['Scenario'] != 'FAOStat']
                data_temp['value'] = data_temp['price'] * data_temp['quantity']
                data_price = (data_temp.groupby(['Period', 'Scenario', 'Model'])['value'].sum() /
                             data_temp.groupby(['Period', 'Scenario', 'Model'])['quantity'].sum()).reset_index()
                data_price = data_price.rename(columns={0: 'Data'})
                data_price['Region'] = 'World'
                data_price['Parameter'] = 'wtRoundwoodPrice'

                # compute roundwood harvest
                data_quantity = data_temp.groupby(['RegionCode', 'Period', 'Scenario', 'Model'])['quantity'].sum().reset_index()
                data_quantity = data_quantity.merge(country_data[['Country-Code', 'ContinentValidation']], left_on='RegionCode',
                                                    right_on='Country-Code', how='left')
                data_quantity = data_quantity.rename(columns={'quantity': 'Data'})
                data_quantity = data_quantity.groupby(['Model', 'Scenario', 'ContinentValidation', 'Period'])['Data'].sum().reset_index()
                data_quantity = data_quantity.rename(columns={'ContinentValidation': 'Region'})
                data_quantity['Parameter'] = 'RoundwoodHarvest'

                data_fin = pd.concat([data_price, data_quantity], axis=0).reset_index(drop=True)

            # filter forest
            if data_key == 'Forest':
                data_temp = data[data_key]
                data_temp = data_temp.drop_duplicates().reset_index(drop=True)
                data_temp['Parameter'] = 'ForestArea'
                data_temp = data_temp.rename(columns={'ForArea': 'Data'})

                data_fin = data_temp[['Model', 'Parameter', 'Scenario', 'RegionCode', 'Period', 'Data']]

            # Aggregation 
            if 'RegionCode' in data_fin.columns:
                data_fin = data_fin.merge(country_data[['Country-Code', 'ContinentValidation']], left_on='RegionCode',
                                           right_on='Country-Code', how='left')
                
                data_fin = data_fin.groupby(['Model', 'Parameter', 'Scenario', 'ContinentValidation', 'Period'])['Data'].sum().reset_index()
                data_fin = data_fin.rename(columns={'ContinentValidation': 'Region'})


            data_filtered = pd.concat([data_filtered, data_fin], axis=0)

        return data_filtered
    
    def reformate_external_data(self, data: pd.DataFrame):
        data = data[data['RCP-SSP'].str.contains('Baseline')].reset_index(drop=True)
        periods = data.columns[6:11]  # select data until 2055
        data_reformated = pd.DataFrame()
        for parameter in data['Estimate'].unique()[:-1]:
            data_temp = data[data['Estimate'] == parameter].reset_index(drop=True)
            data_info = data_temp.iloc[:, : 6]
            data_periods = pd.DataFrame()
            for period in periods:
                data_period = data_temp[period]
                data_period = pd.DataFrame(data_period).rename(columns={period: 'Data'})
                data_period['Period'] = int(period)
                data_period = pd.concat([data_info, data_period], axis=1)
                data_periods = pd.concat([data_periods, data_period], axis=0)

            data_reformated = pd.concat([data_reformated, data_periods], axis=0)
        data_reformated = data_reformated.rename(columns={'Estimate': 'Parameter',
                                                           'SSP': 'Scenario'})
        data_reformated = data_reformated[['Model', 'Parameter', 'Scenario', 'Region', 'Period', 'Data']]

        return data_reformated

    def period_structure(self, data: pd.DataFrame):
        data = data[data['Model'] == 'GFPMpt']
        period_data = data[['Period', 'year']].drop_duplicates().reset_index(drop=True)
        return period_data
    
    def align_period_data(self, data: pd.DataFrame, period_data: pd.DataFrame):
        data = data.merge(period_data, left_on='Period', right_on='Period', how='left')
        period_len = 10
        period_list = range(2015, 2055, period_len)
        data_aligned = pd.DataFrame()

        for period in period_list:
            """if period == 2015:  # Baseperiod
                data_period = data[data['year'] == period]
                data_period_index = data[data['year'] == period].index
                data_period.loc[data_period_index, 'Period'] = data_period.loc[data_period_index, 'year']
                data_period = data_period[['Model', 'Parameter', 'Scenario', 'Region', 'Period', 'Data']]"""

            if period == 2025:  # Mixed period with single- and multiple-year-period 
                data_single_year_period = data[[x in list(range(period - period_len, period))[1:] for x in data['year']]]
                data_multiple_year_period = data[data['year'] == period]

                data_single_year_period = data_single_year_period.groupby(['Model', 'Parameter', 'Scenario', 'Region'])['Data'].sum().reset_index()
                data_multiple_year_period = data_multiple_year_period.groupby(['Model', 'Parameter', 'Scenario', 'Region'])['Data'].sum().reset_index()
                data_multiple_year_period.loc[data_multiple_year_period.index,'Data'] = data_multiple_year_period.loc[data_multiple_year_period.index,'Data'] * 5
                
                data_single_year_period = data_single_year_period.reset_index(drop=True)
                data_multiple_year_period = data_multiple_year_period.reset_index(drop=True)
                
                data_single_year_period['Data'] = (data_single_year_period['Data'] + data_multiple_year_period['Data']) / 10
                data_single_year_period['Period'] = period
                data_period = data_single_year_period.reset_index(drop=True)                                             
            else:  # all other multiple-year-periods
                data_period = data[data['year'] == period]
                data_period_index = data[data['year'] == period].index
                data_period.loc[data_period_index, 'Period'] = data_period.loc[data_period_index, 'year']
                data_period = data_period[['Model', 'Parameter', 'Scenario', 'Region', 'Period', 'Data']].reset_index(drop=True)
                
            data_aligned = pd.concat([data_aligned, data_period], axis=0).reset_index(drop=True)

        return data_aligned
    
    def add_world(self, data: pd.DataFrame):

        data = data[data['Region'] != 'Rest of World'].reset_index(drop=True)
        data_new = pd.DataFrame()

        list_parameter = ['CarbonStockBiomass [MtCO2]', 'CarbonStockHWP [MtCO2]', 'ForestArea', 'RoundwoodHarvest']

        for parameter in data['Parameter'].unique():
            data_temp = data[data['Parameter'] == parameter]
            
            if parameter in list_parameter:
                data_world = data_temp.groupby(['Model', 'Parameter', 'Scenario', 'Period'])['Data'].sum().reset_index()
                data_world['Region'] = 'World'
                data_temp = pd.concat([data_temp, data_world], axis=0).reset_index(drop=True)
            data_new = pd.concat([data_new, data_temp], axis=0).reset_index(drop=True)
        
        return data_new


    
    def convert_unit(self, data: pd.DataFrame):

        # Convert harvest (tsd m³ to M m³):
        col_name = 'RoundwoodHarvest'
        selection_index = data[data['Parameter'] == col_name].index
        data.loc[selection_index, 'Data'] = data.loc[selection_index, 'Data'] / 1000

        # Convert forest area (tsd ha to M ha)
        col_name = 'ForestArea'
        selection_index = data[data['Parameter'] == col_name].index
        data.loc[selection_index, 'Data'] = data.loc[selection_index, 'Data'] / 1000

        # Convert carbon (CO2 to C):
        col_name = ['CarbonStockBiomass [MtCO2]', 'CarbonStockHWP [MtCO2]']
        selection_index = data[[x in col_name for x in data['Parameter']]].index
        data.loc[selection_index, 'Data'] = data.loc[selection_index, 'Data'] / (44/12)

        return data

    def rename_parameter(self, data: pd.DataFrame):
        # rename parameter
        parameter_name = {'CarbonStockBiomass [MtCO2]': 'Total Forest Non-soil C Stock (MtC)',
                          'ForestArea': 'Forest Area (Mha)',
                          'RoundwoodHarvest': 'Roundwood Harvest (Mm3/yr)',
                          'wtRoundwoodPrice': 'Wt Avg Roundwood Price (/m3)'}
        
        for parameter in parameter_name.keys():
            selection_index = data[data['Parameter'] == parameter].index
            data.loc[selection_index, 'Parameter'] = parameter_name[parameter]
        
        # rename scenario
        data['Scenario'] = data['Scenario'].str.split('_', expand=True)[5]

        return data

    def merge_data(self, data: pd.DataFrame, external_data: pd.DataFrame):
        data_fin = pd.concat([data, external_data], axis=0).reset_index(drop=True)
        
        return data_fin


import click
import warnings
from Toolbox.toolbox import timba_dashboard,validation_dashboard
import Toolbox.parameters.paths as toolbox_paths
from pathlib import Path
warnings.simplefilter(action='ignore', category=FutureWarning)

@click.group()
def cli():
    pass

#Dashboard Command
@click.command()
@click.option('-NF', '--num_files', default=10, 
              show_default=True, required=True, type=int, 
              help="Number of .pkl files to read")
@click.option('-FP', '--sc_folderpath', default=toolbox_paths.SCINPUTPATH, 
              show_default=True, required=True, type=Path, 
              help="Define the folder where the code will look for .pkl files containing the scenarios.")
@click.option('-AIFP', '--additional_info_folderpath', default=toolbox_paths.AIINPUTPATH, 
              show_default=True, required=True, type=Path, 
              help="Define the folder where the code will look for additional infos, like historic data or country information.")
@click.option('-PS', '--print_settings', default=False, 
              show_default=True, required=True, type=bool, 
              help="If True, the quantity plot will be plotted with bigger lines and font sizes, so that the figure can be used for presentations etc.")

def dashboard_cli(num_files,sc_folderpath,additional_info_folderpath,print_settings):    
    td = timba_dashboard(num_files_to_read=num_files,
                         scenario_folder_path=sc_folderpath,
                         additional_info_folderpath=additional_info_folderpath,
                         print_settings=print_settings)
    td.run()

#Validation Command
@click.command()
@click.option('-NF', '--num_files', default=10, 
              show_default=True, required=True, type=int, 
              help="Number of .pkl files to read")
@click.option('-FP', '--sc_folderpath', default=toolbox_paths.SCINPUTPATH, 
              show_default=True, required=True, type=Path, 
              help="Folder path for scenarios")
@click.option('-AIFP', '--additional_info_folderpath', default=toolbox_paths.AIINPUTPATH, 
              show_default=True, required=True, type=Path, 
              help="Define the folder where the code will look for additional infos, like historic data or country information.")
def validation_cli(num_files, sc_folderpath,additional_info_folderpath):    
    click.echo("Validation is started")
    validb = validation_dashboard(
        num_files_to_read=num_files,
        scenario_folder_path=sc_folderpath,
        additional_info_folderpath=additional_info_folderpath
    )
    validb.run()

cli.add_command(dashboard_cli, name="dashboard")
cli.add_command(validation_cli, name="validation")




import plotly.graph_objects as go
import dash
import dash_bootstrap_components as dbc
from dash import dcc, html
from dash.dependencies import Input, Output, State
import webbrowser
from threading import Timer
import pandas as pd
import numpy as np
from pathlib import Path
import plotly.express as px
import textwrap
import re

PACKAGEDIR = Path(__file__).parent.parent.absolute()


class Vali_DashboardPlotter:

    def __init__(self, data):
        self.data = data
        self.app = dash.Dash(__name__, external_stylesheets=[dbc.themes.BOOTSTRAP])
        self.start = self.data['Year'].min()
        self.end = self.data['Year'].max()
        self.model_colors = self.get_colors()
        self.logo = PACKAGEDIR / 'timba_validation_logo.png'
        self.create_layout()
        self.create_callbacks()

    def get_colors(self):
        color_palette = px.colors.qualitative.Bold
        model_colors = {
            "TiMBA": tuple(map(int, re.findall(r'\d+', color_palette[1]))),
            "GLOBIOM": tuple(map(int, re.findall(r'\d+', color_palette[4]))),
            "GFPM": tuple(map(int, re.findall(r'\d+', color_palette[3]))),
            "GTM": tuple(map(int, re.findall(r'\d+', color_palette[2]))),
            "Max": tuple(map(int, re.findall(r'\d+', color_palette[6]))),
            "Min": tuple(map(int, re.findall(r'\d+', color_palette[8]))),
        }

        return model_colors

    def create_layout(self):
        dropdown_style = {
            'height': '30px',
            'marginRight': '10px',
            'flex': '1 1 200px',  # Flex-Wachstum mit Basisgröße
            'minWidth': '200px'   # Mindestbreite für bessere Lesbarkeit
        }

        self.app.layout = dbc.Container(fluid=True, style={'backgroundColor': 'white'}, children=[

            # 1. filter options (bottom line)
            dbc.Row([
                dbc.Col([
                    dbc.Card(className="border-0 shadow-sm", children=[
                        dbc.CardBody(style={'padding': '15px'}, children=[
                            html.Div(style={
                                'display': 'flex',
                                'flexWrap': 'wrap',
                                'gap': '10px',
                                'alignItems': 'center'
                            }, children=[
                                # Logo links
                                html.Img(
                                    src=self.app.get_asset_url('timba_validation_logo.png'),
                                    style={'height': '50px', 'marginRight': '20px'}
                                ),

                                # Filter-Dropdowns
                                dcc.Dropdown(id='region-dropdown',
                                             options=[{'label': i, 'value': i}
                                                      for i in sorted(self.data['Region'].dropna().unique())],
                                             multi=True,
                                             placeholder='Select Region...',
                                             style=dropdown_style),
                                dcc.Dropdown(id='estimate-dropdown',
                                             options=[{'label': i, 'value': i}
                                                      for i in sorted(self.data['Estimate'].dropna().unique())],
                                             placeholder='Select Estimate...',
                                             multi=True,
                                             style=dropdown_style),
                                dcc.Dropdown(id='scenario-dropdown',
                                             options=[{'label': 'All', 'value': 'All'}] +
                                                     [{'label': i, 'value': i}
                                                      for i in sorted(self.data['Scenario'].dropna().unique())],
                                             placeholder='Select Scenario...',
                                             multi=True,
                                             style=dropdown_style),
                                dcc.Dropdown(id='model-dropdown',
                                             options=[{'label': 'All', 'value': 'All'}] +
                                                     [{'label': i, 'value': i}
                                                      for i in sorted(self.data['Model'].dropna().unique())],
                                             placeholder='Select Model...',
                                             multi=True,
                                             style=dropdown_style),
                                # Download-Button
                                html.Button(
                                    "⬇️ CSV Export",
                                    id="btn_csv",
                                    className="ml-auto",
                                    style={
                                        'height': '30px',
                                        'marginLeft': 'auto',
                                        'padding': '0 15px',
                                        'borderRadius': '4px',
                                        'border': '1px solid #ddd'
                                    }
                                ),
                                dcc.Download(id="download-dataframe-csv")
                            ])
                        ])
                    ], style={'backgroundColor': '#f8f9fa'})
                ])
            ], className="mb-4"),  # Distance to bottom

            # 2. Main content
            dbc.Row([
                # Left column
                dbc.Col(children=[
                    dbc.Card(className="h-100 shadow-sm", children=[
                        dbc.CardBody(children=[
                            # html.H5("Figure filter", className="card-title"),
                            html.Div([dcc.Dropdown(
                                id='figure-type-dropdown',
                                options=[{'label': i, 'value': i}
                                                      for i in ['range', 'min_max', 'ssp_fsm_range', 'ssp_fsm_all']],
                                placeholder="Select Figure Type...",
                                value='ssp_fsm_range',
                                style=dropdown_style
                            )], style={'marginBottom': '15px'}),
                            dcc.Graph(
                                id='formip-plot',
                                config={'toImageButtonOptions': {'format': 'png'},
                                        'displayModeBar': True},
                                style={'height': '75vh'}
                            )
                        ])
                    ], style={'backgroundColor': 'white', 'padding': '15px'})
                ], width=6),

                dbc.Col(md=6, children=[
                    # Right column
                    dbc.Card(className="h-100 shadow-sm", children=[
                        dbc.CardBody(style={'padding': '15px'}, children=[
                            # html.H5("Figure filter", className="card-title"),
                            html.Div([
                                dcc.Dropdown(
                                    id='value-type-dropdown',
                                    options=[{'label': i, 'value': i} for i in ['relative values', 'absolute values']],
                                    placeholder="Select Value Type...",
                                    value='relative values',
                                    style=dropdown_style
                                )
                            ], style={'marginBottom': '15px'}),

                            html.Div([
                                dcc.Dropdown(
                                    id='start-year-dropdown',
                                    options=[{'label': i, 'value': i} for i in
                                             sorted(self.data['Year'].dropna().unique())],
                                    placeholder="Select Start Year ...",
                                    value=2020,
                                    style=dropdown_style
                                )
                            ], style={'marginBottom': '15px'}),

                            html.Div([
                                dcc.Dropdown(
                                    id='end-year-dropdown',
                                    options=[{'label': i, 'value': i} for i in
                                             sorted(self.data['Year'].dropna().unique())],
                                    placeholder="Select End Year ...",
                                    value=2100,
                                    style=dropdown_style
                                )
                            ], style={'marginBottom': '15px'}),
                            dcc.Graph(
                                id='formip-bar',
                                config={'toImageButtonOptions': {'format': 'png'},
                                        'displayModeBar': True},
                                style={'height': '75vh'}
                            )
                        ])
                    ], style={'backgroundColor': 'white', 'padding': '15px'})
                ], width=6)
            ])
        ])

    def create_callbacks(self):
        @self.app.callback([
                Output('formip-plot', 'figure'),
                Output('formip-bar', 'figure')],
            [
                Input('region-dropdown', 'value'),
                Input('estimate-dropdown', 'value'),
                Input('scenario-dropdown', 'value'),
                Input('model-dropdown', 'value'),
                Input('figure-type-dropdown', 'value'),
                Input('value-type-dropdown', 'value'),
                Input('start-year-dropdown', 'value'),
                Input('end-year-dropdown', 'value')]
        )
        def update_plots(region, estimate, scenario, model, figure_type, value_type, start_year, end_year):
            return self.update_plot_validation(region, estimate, scenario, model, figure_type, value_type, start_year,
                                               end_year)

        @self.app.callback(
            Output("download-dataframe-csv", "data"),
            Input("btn_csv", "n_clicks"),
            [State('region-dropdown', 'value'),
             State('estimate-dropdown', 'value'),
             State('scenario-dropdown', 'value'),
             State('model-dropdown', 'value')],
            prevent_initial_call=True
        )
        def func(n_clicks, region, estimate, scenario, model):
            if n_clicks is None:
                raise dash.exceptions.PreventUpdate

            filtered_data = self.filter_data(region, estimate, scenario, model)
            return dcc.send_data_frame(filtered_data.to_csv, "filtered_external_validation_data.csv", index=False)

    def filter_data(self, region, estimate, scenario, model):
        filtered_data = self.data.copy()
        if isinstance(region, list):
            if not region:
                filtered_data = filtered_data.iloc[0:0]
            else:
                filtered_data = filtered_data[filtered_data['Region'].isin(region)]
        if isinstance(model, list):
            if not model:
                filtered_data = filtered_data.iloc[0:0]
            elif "All" not in model:
                filtered_data = filtered_data[filtered_data['Model'].isin(model)]
        if isinstance(estimate, list):
            if not estimate:
                filtered_data = filtered_data.iloc[0:0]
            else:
                filtered_data = filtered_data[filtered_data['Estimate'].isin(estimate)]
        if isinstance(scenario, list):
            if not scenario:
                filtered_data = filtered_data.iloc[0:0]
            elif "All" not in scenario:
                filtered_data = filtered_data[filtered_data['Scenario'].isin(scenario)]
        return filtered_data

    def plot_min_max(self, data):
        fig_formip_main = go.Figure()

        for (model, region), subset in data.groupby(['Model', 'Region']):
            color = self.model_colors.get(model, (0, 0, 0))
            dash = 'solid'

            if model == 'Max':
                name_up_bnd = 'Upper bound max'
                name_low_bnd = 'Confidence interval max'
                name_mean = 'Mean max'
            if model == 'Min':
                name_up_bnd = 'Upper bound min'
                name_low_bnd = 'Confidence interval min'
                name_mean = 'Mean min'
            if model == 'TiMBA':
                name_up_bnd = 'Upper bound TiMBA'
                name_low_bnd = 'Confidence interval TiMBA'
                name_mean = 'Mean TiMBA'

            subset_new = pd.DataFrame()
            for year in subset['Year'].unique():
                subset_tmp = subset[subset['Year'] == year].reset_index(drop=True)
                subset_tmp_info = pd.DataFrame([subset_tmp.iloc[0, 0: -1]])
                subset_tmp_mean = pd.DataFrame([subset_tmp["Data"].mean()], columns=['mean'])
                subset_tmp_max = pd.DataFrame([subset_tmp["Data"].max()], columns=['max'])
                subset_tmp_min = pd.DataFrame([subset_tmp["Data"].min()], columns=['min'])

                subset_tmp_new = pd.concat(
                    [subset_tmp_info, subset_tmp_mean, subset_tmp_max, subset_tmp_min], axis=1).reset_index(drop=True)
                subset_new = pd.concat([subset_new, subset_tmp_new], axis=0)

            # Add upper bound
            fig_formip_main.add_trace(go.Scatter(
                x=subset_new['Year'],
                y=subset_new['max'],
                line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", width=0),
                mode='lines',
                showlegend=False,
                name=name_up_bnd
            ))

            # Add lower bound and fill to upper
            fig_formip_main.add_trace(go.Scatter(
                x=subset_new['Year'],
                y=subset_new['min'],
                fill='tonexty',
                fillcolor=f"rgba({color[0]}, {color[1]}, {color[2]}, 0.2)",
                line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", width=0),
                mode='lines',
                showlegend=True,
                name=name_low_bnd
            ))

            # Add the mean line
            fig_formip_main.add_trace(go.Scatter(
                x=subset_new['Year'],
                y=subset_new['mean'],
                line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", dash=dash),
                mode='lines',
                name=name_mean
            ))
        return fig_formip_main

    def plot_range(self, formip_data, timba_data):
        fig_formip_main = go.Figure()

        color_range = self.model_colors.get("", (0, 0, 0))
        dash = 'solid'
        formip_data = (formip_data.groupby('Year').agg(
            mean=('Data', 'mean'),
            min=('Data', 'min'),
            max=('Data', 'max')).reset_index())

        formip_data['lower_err'] = formip_data['mean'] - formip_data['min']
        formip_data['upper_err'] = formip_data['max'] - formip_data['mean']

        # Add upper bound ForMIP range
        fig_formip_main.add_trace(go.Scatter(
            x=formip_data['Year'],
            y=formip_data['max'],
            line=dict(color=f"rgba({color_range[0]}, {color_range[1]}, {color_range[2]}, 1)", width=0),
            mode='lines',
            showlegend=False,
            name="Upper bound"
        ))

        # Add lower bound ForMIP range
        fig_formip_main.add_trace(go.Scatter(
            x=formip_data['Year'],
            y=formip_data['min'],
            fill='tonexty',
            fillcolor=f"rgba({color_range[0]}, {color_range[1]}, {color_range[2]}, 0.2)",
            line=dict(color=f"rgba({color_range[0]}, {color_range[1]}, {color_range[2]}, 1)", width=0),
            mode='lines',
            showlegend=True,
            name="ForMIP Min-Max range"
        ))

        color = self.model_colors.get("TiMBA", (0, 0, 0))
        timba_data = (timba_data.groupby('Year').agg(
            mean=('Data', 'mean'),
            min=('Data', 'min'),
            max=('Data', 'max')).reset_index())

        timba_data['lower_err'] = timba_data['mean'] - timba_data['min']
        timba_data['upper_err'] = timba_data['max'] - timba_data['mean']

        # Add upper bound TiMBA
        fig_formip_main.add_trace(go.Scatter(
            x=timba_data['Year'],
            y=timba_data['max'],
            line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", width=0),
            mode='lines',
            showlegend=False,
            name="Upper bound"
        ))

        # Add lower bound TiMBA
        fig_formip_main.add_trace(go.Scatter(
            x=timba_data['Year'],
            y=timba_data['min'],
            fill='tonexty',
            fillcolor=f"rgba({color[0]}, {color[1]}, {color[2]}, 0.2)",
            line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", width=0),
            mode='lines',
            showlegend=True,
            name="TiMBA Min-Max range"
        ))

        # Add the mean line TiMBA
        fig_formip_main.add_trace(go.Scatter(
            x=timba_data['Year'],
            y=timba_data['mean'],
            line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", dash=dash),
            mode='lines',
            name="TiMBA mean"
        ))
        return fig_formip_main

    def plot_ssp_fsm_range(self, data):
        fig_formip_main = go.Figure()

        for (model, region), subset in data.groupby(['Model', 'Region']):
            color = self.model_colors.get(model, (0, 0, 0))
            dash = 'solid'

            name_up_bnd = f'Upper bound {model}'
            name_low_bnd = f'Min-Max range {model}'
            name_mean = f'Mean {model}'

            subset_new = pd.DataFrame()
            for year in subset['Year'].unique():
                subset_tmp = subset[subset['Year'] == year].reset_index(drop=True)
                subset_tmp_info = pd.DataFrame([subset_tmp.iloc[0, 0: -1]])
                subset_tmp_mean = pd.DataFrame([subset_tmp["Data"].mean()], columns=['mean'])
                subset_tmp_max = pd.DataFrame([subset_tmp["Data"].max()], columns=['max'])
                subset_tmp_min = pd.DataFrame([subset_tmp["Data"].min()], columns=['min'])

                subset_tmp_new = pd.concat(
                    [subset_tmp_info, subset_tmp_mean, subset_tmp_max, subset_tmp_min], axis=1).reset_index(drop=True)
                subset_new = pd.concat([subset_new, subset_tmp_new], axis=0)

            # Add upper bound
            fig_formip_main.add_trace(go.Scatter(
                x=subset_new['Year'],
                y=subset_new['max'],
                line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", width=0),
                mode='lines',
                showlegend=False,
                name=name_up_bnd
            ))

            # Add lower bound and fill to upper
            fig_formip_main.add_trace(go.Scatter(
                x=subset_new['Year'],
                y=subset_new['min'],
                fill='tonexty',
                fillcolor=f"rgba({color[0]}, {color[1]}, {color[2]}, 0.2)",
                line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", width=0),
                mode='lines',
                showlegend=True,
                name=name_low_bnd
            ))

            # Add the mean line
            fig_formip_main.add_trace(go.Scatter(
                x=subset_new['Year'],
                y=subset_new['mean'],
                line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", dash=dash),
                mode='lines',
                name=name_mean
            ))
        return fig_formip_main

    def plot_ssp_fsm_all(self, data):
        fig_formip_main = go.Figure()
        dash_styles = {}
        dash_list = ["solid", "dash", "dot", "longdash", "dashdot", "longdashdot"]
        dash_runner = 0
        for scenario in data["Scenario"].unique():
            if dash_runner > 5:
                dash_runner = 0

            dash_styles[scenario] = dash_list[dash_runner]

            dash_runner += 1
        
        for (model, region, scenario), subset in data.groupby(['Model', 'Region', 'Scenario']):
            color = self.model_colors.get(model, (0, 0, 0))
            name_mean = f'{model}_{scenario}'

            subset_new = pd.DataFrame()
            for year in subset['Year'].unique():
                subset_tmp = subset[subset['Year'] == year].reset_index(drop=True)
                subset_tmp_info = pd.DataFrame([subset_tmp.iloc[0, 0: -1]])
                subset_tmp_mean = pd.DataFrame([subset_tmp["Data"].mean()], columns=['mean'])

                subset_tmp_new = pd.concat(
                    [subset_tmp_info, subset_tmp_mean], axis=1).reset_index(drop=True)
                subset_new = pd.concat([subset_new, subset_tmp_new], axis=0)

            # Add the mean line
            fig_formip_main.add_trace(go.Scatter(
                x=subset_new['Year'],
                y=subset_new['mean'],
                line=dict(color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)", dash=dash_styles[scenario]),
                mode='lines',
                name=name_mean
            ))

        return fig_formip_main

    def bar_plot_fsm(self, data, value_type, start_year, end_year):
        fig_formip_second = go.Figure()

        data_grouped = data.groupby(['Model', 'Region'])

        for (model, region), subset in data_grouped:
            color = self.model_colors.get(model, (0, 0, 0))
            scenario_diffs = []

            for scenario in subset['Scenario'].unique():
                scenario_data = subset[subset['Scenario'] == scenario]

                try:
                    data_start_year = float(scenario_data[scenario_data['Year'] == start_year]['Data'])
                    data_end_year = float(scenario_data[scenario_data['Year'] == end_year]['Data'])

                    if value_type == 'absolute values':
                        data_diff = data_end_year - data_start_year
                    elif value_type == 'relative values':
                        data_diff = (data_end_year - data_start_year) / data_end_year * 100

                    scenario_diffs.append(data_diff)
                except Exception:
                    continue

            if len(scenario_diffs) == 0:
                continue  # No valid data

            mean_diff = np.mean(scenario_diffs)
            min_diff = np.min(scenario_diffs)
            max_diff = np.max(scenario_diffs)

            error_plus = max_diff - mean_diff
            error_minus = mean_diff - min_diff

            fig_formip_second.add_trace(go.Bar(
                x=[f'{model}<br>{start_year} - {end_year}'],
                y=[mean_diff],
                name=model,
                marker_color=f"rgba({color[0]}, {color[1]}, {color[2]}, 0.6)",
                error_y=dict(
                    type='data',
                    symmetric=False,
                    array=[error_plus],  # Upper CI
                    arrayminus=[error_minus],  # Lower CI
                    thickness=1.5,
                    width=5,
                    color=f"rgba({color[0]}, {color[1]}, {color[2]}, 1)"
                )
            ))

        return fig_formip_second

    def update_plot_validation(self, region, estimate, scenario, model, figure_type, value_type, start_year, end_year):
        graphic_template = 'plotly_white'  # 'plotly_dark'#'plotly_white'
        filtered_data = self.filter_data(region, estimate, scenario, model)

        filtered_data = filtered_data.reset_index(drop=True)
        filtered_data['Model'] = filtered_data['Model'].str.strip()

        # ForMIP main plot (absolute value comparison)

        if (figure_type == 'min_max') | (figure_type == 'range'):
            timba_data = filtered_data[filtered_data['Model'] == 'TiMBA'].reset_index(drop=True)

            fsm_data = filtered_data[filtered_data['Model'] != 'TiMBA'].reset_index(drop=True)
            fsm_data_max = fsm_data.groupby(['Year', 'Region', 'Estimate', 'Scenario'])['Data'].max().reset_index()
            fsm_data_min = fsm_data.groupby(['Year', 'Region', 'Estimate', 'Scenario'])['Data'].min().reset_index()
            fsm_data_max['Model'] = 'Max'
            fsm_data_min['Model'] = 'Min'

            if figure_type == 'min_max':
                data_fin = pd.concat([timba_data, fsm_data_max, fsm_data_min], axis=0).reset_index(drop=True)
                fig_formip_main = self.plot_min_max(data=data_fin)

            else:
                data_fin = pd.concat([fsm_data_max, fsm_data_min], axis=0).reset_index(drop=True)
                fig_formip_main = self.plot_range(formip_data=data_fin, timba_data=timba_data)

        elif figure_type == "ssp_fsm_range":
            fig_formip_main = self.plot_ssp_fsm_range(data=filtered_data)

        elif figure_type == "ssp_fsm_all":
            fig_formip_main = self.plot_ssp_fsm_all(data=filtered_data)

        else:
            fig_formip_main = go.Figure()

        title_formip_main = self.generate_title(
            region=region, estimate=estimate, scenario=scenario, model=model, plot="plot", value_type=value_type)

        fig_formip_main.update_layout(
            title='<br>'.join(textwrap.wrap(title_formip_main, width=150)),
            xaxis_title='Year',
            #yaxis_title=f'{estimate[0]}',
            xaxis=dict(gridcolor='white'),
            yaxis=dict(rangemode='nonnegative', zeroline=True, zerolinewidth=2, zerolinecolor='LightGrey',
                       gridcolor='white'),
            legend=dict(orientation="h", yanchor="top", y=-0.1, xanchor="center", x=0.5),
            margin=dict(l=35, r=35, t=110, b=90),
            hovermode='x unified',
            template=graphic_template,
            plot_bgcolor='rgb(229, 236, 246)'
        )

        # ForMIP secondary plot (relative value comparison)
        fig_formip_second = self.bar_plot_fsm(data=filtered_data,
                                              value_type=value_type,
                                              start_year=start_year,
                                              end_year=end_year)

        title_formip_second = self.generate_title(
            region=region, estimate=estimate, scenario=scenario, model=model, plot="bar_plot", value_type=value_type)
        if value_type == "absolute values":
            pass#yaxis_title = f'Difference in {estimate[0]}'
        if value_type == "relative values":
            pass#yaxis_title = f'Difference in {' '.join(estimate[0].split(' ')[:-1])} (%)'

        fig_formip_second.update_layout(
            title='<br>'.join(textwrap.wrap(title_formip_second, width=150)),
            #yaxis_title=yaxis_title,
            xaxis=dict(gridcolor='white'),
            yaxis=dict(zeroline=True, zerolinewidth=2, zerolinecolor='LightGrey', gridcolor='white'),
            legend=dict(orientation="h", yanchor="top", y=-0.1, xanchor="center", x=0.5),
            margin=dict(l=35, r=35, t=135, b=100),
            hovermode='x unified',
            template=graphic_template,
            plot_bgcolor='rgb(229, 236, 246)'
        )

        return fig_formip_main, fig_formip_second


    def generate_title(self, region, estimate, scenario, model, plot, value_type):
        title_parts = []
        if plot=="bar_plot":
            if value_type == "absolute values":
                pass#title_parts.append(f'Difference in {estimate[0]}')
            if value_type == "relative values":
                pass#title_parts.append(f'Difference in {' '.join(estimate[0].split(' ')[:-1])} (%)')
        else:
            title_parts.append(f"{estimate}")
        if region:
            title_parts.append(f" for {region}<br>")
        if scenario:
            title_parts.append(f"Scenarios: {scenario}<br>")
        if model:
            title_parts.append(f"Models: {model}")
        title = "".join(title_parts) if title_parts else "all data"
        clean_title = title.replace("'", "").replace("[", "").replace("]", "")
        return clean_title

    def open_browser(self):
        webbrowser.open_new("http://localhost:8050")

    def run(self):
        Timer(1, self.open_browser).start()
        self.app.run(host='localhost', debug=False, dev_tools_ui=False, dev_tools_hot_reload=False, port=8050)



import pandas as pd

fao = pd.read_csv('Forestry_subsetted_Data_reformatted.csv')

h_commodity = pd.read_csv('commodity_info.csv', encoding="ISO-8859-1")
h_commodity = h_commodity[['GFPM_Code','FAO-Code']]
h_commodity.columns = ['GFPM_Code_Commodity','FAO_Code_commodity']

h_country = pd.read_csv('country_info.csv', encoding="ISO-8859-1")
h_country = h_country[['Country-Code', 'FAOCou-Code']]
h_country.columns = ['GFPM_Code_Country','FAO_Code_Country']

fao_commodity = fao.merge(right=h_commodity, how='left',left_on='Item_Codes',right_on='FAO_Code_commodity').dropna()
fao_final = fao_commodity.merge(right=h_country, how='left',left_on='Area_Codes',right_on='FAO_Code_Country').dropna()
fao_final = fao_final.reset_index(drop=True)

fao_final_o5 = fao_final[fao_final.GFPM_Code_Country == 'o5']
print(fao_final_o5)

fao_final.to_csv('FAO_Data_py.csv')


seperator_scenario_name = "Sc_"
column_name_scenario = "Scenario"
column_name_model = "Model"
column_name_id = "ID"
model_name = "TiMBA"

default_plot_settings = {"line_witdh":1.5,
                         "title_font_size":20,
                         "font_size":16,
                         "tick_font_size":14,
                         "legend_font_size":14}

printing_plot_settings = {"line_witdh":7,
                          "title_font_size":32,
                          "font_size":26,
                          "tick_font_size":24,
                          "legend_font_size":18}


import pytest
import pandas as pd
from unittest.mock import patch, mock_open
import pickle
import gzip
from pathlib import Path
from io import BytesIO
from Toolbox.classes.scenario_plots import PlotDropDown, sc_plot
import Toolbox.parameters.paths as toolbox_paths
import Toolbox.parameters.default_parameters as toolbox_parameters

@pytest.fixture
def sample_data():
    return pd.DataFrame({
        'year': [2000, 2001, 2002, 2000, 2001, 2002],
        'Scenario': ['A', 'A', 'A', 'B', 'B', 'B'],
        'quantity': [10, 20, 30, 15, 25, 35],
        'RegionCode': ['R1'] * 6,
        'Continent': ['Europe'] * 6,
        'Model': ['M1'] * 6,
        'domain': ['Dom1'] * 6,
        'CommodityCode': ['C1'] * 6,
    })

def test_predefined_plot_runs_without_error(sample_data):
    plot_instance = sc_plot()
    try:
        plot_instance.predefined_plot(sample_data)
    except Exception as e:
        pytest.fail(f"predefined_plot() raised an exception: {e}")

def test_plot_dropdown_initialization(sample_data):
    try:
        dropdown = PlotDropDown(sample_data)
        assert dropdown.data.equals(sample_data)
    except Exception as e:
        pytest.fail(f"PlotDropDown init failed: {e}")




from enum import Enum

class CountryGroups(Enum):
    formip_regions = {
        "Africa": [
            "AGO", "ARE", "BDI", "BEN", "BFA", "BHR", "BWA", "CAF", "CIV", "CMR", "COD", "COG", "COM", "CPV", "DJI",
            "DZA", "EGY", "ERI", "ESH", "ETH", "GAB", "GHA", "GIN", "GMB", "GNB", "GNQ", "IRN", "IRQ", "ISR", "JOR",
            "KEN", "KWT", "LBN", "LBR", "LBY", "LSO", "MAR", "MDG", "MLI", "MOZ", "MRT", "MUS", "MWI", "MYT", "NAM",
            "NER", "NGA", "OMN", "PSE", "QAT", "REU", "RWA", "SAU", "SDN", "SEN", "SHN", "SLE", "SOM", "SSD", "STP",
            "SWZ", "SYC", "SYR", "TCD", "TGO", "TUN", "TUR", "TZA", "UGA", "YEM", "ZAF", "ZMB", "ZWE"
        ],
        "Asia": [
            "AFG", "ASM", "AUS", "BGD", "BRN", "BTN", "CHN", "COK", "FJI", "FSM", "GUM", "IDN", "IND", "JPN", "KHM",
            "KIR", "KOR", "LAO", "LKA", "MDV", "MHL", "MMR", "MNG", "MNP", "MYS", "NCL", "NFK", "NIU", "NPL", "NRU",
            "NZL", "PAK", "PCN", "PHL", "PLW", "PNG", "PRK", "PYF", "SGP", "SLB", "THA", "TKL", "TLS", "TON", "TUV",
            "VNM", "VUT", "WLF", "WSM"
        ],
        "Europe": [
            "ALB", "AND", "AUT", "BGR", "BIH", "BEL", "CHE", "CZE", "DEU", "DNK", "ESP", "EST", "FIN", "FRA", "FRO",
            "GBR", "GGY", "GIB", "GRC", "HRV", "HUN", "IMN", "IRL", "ISL", "ITA", "JEY", "LIE", "LTU", "LUX", "LVA",
            "MCO", "MKD", "MLT", "MNE", "NLD", "NOR", "POL", "PRT", "ROU", "SJM", "SMR", "SRB", "SVK", "SVN", "SWE",
            "VAT"
        ],
        "Former Soviet Union": [
            "ARM", "AZE", "BLR", "GEO", "KAZ", "KGZ", "MDA", "RUS", "TJK", "TKM", "UZB", "UKR"
        ],
        "Latin America": [
            "ABW", "AIA", "ARG", "ATG", "BOL", "BES", "BHS", "BLM", "BLZ", "BMU", "BRA", "BRB", "CHL", "COL", "CRI",
            "CUB", "CUW", "CYM", "DMA", "DOM", "ECU", "FLK", "GLP", "GRD", "GTM", "GUF", "GUY", "HND", "HTI", "JAM",
            "KNA", "LCA", "MAF", "MEX", "MSR", "MTQ", "NIC", "PAN", "PER", "PRI", "PRY", "SLV", "SPM", "SUR", "SXM",
            "TCA", "TTO", "URY", "VCT", "VEN", "VGB", "VIR"
        ],
        "North America": [
            "CAN", "USA", "GRL"
        ]
    }


from pathlib import Path

PACKAGEDIR = Path(__file__).resolve().parent.parent
SCINPUTPATH = PACKAGEDIR / Path("Input") / Path("Scenario_Files")
AIINPUTPATH = PACKAGEDIR / Path("Input") / Path("Additional_Information")
SCFOLDERPATH = Path("Input")
COUNTRYINFO = "country_info.csv"
COMMODITYINFO = "commodity_info.csv"
FORESTINFO = "Forest_world500.csv"
HISTINFO = "FAO_Data.csv"
FORMIP = "external_model_data.csv"


from Toolbox.classes.import_data import import_pkl_data
from Toolbox.classes.dashboard import DashboardPlotter
from Toolbox.classes.validation_db import Vali_DashboardPlotter
from pathlib import Path
import Toolbox.parameters.paths as toolbox_paths

class timba_dashboard:
    def __init__(self,scenario_folder_path:Path,
                 additional_info_folderpath:Path,
                 num_files_to_read:int=10,
                 print_settings:bool=False):
        self.num_files_to_read = num_files_to_read
        self.scenario_folder_path = scenario_folder_path
        self.additional_info_folderpath = additional_info_folderpath
        self.print_settings = print_settings

    def import_data(self):
        import warnings
        warnings.simplefilter(action='ignore', category=FutureWarning)
        import_pkl = import_pkl_data(num_files_to_read=self.num_files_to_read,
                                     SCENARIOPATH=self.scenario_folder_path,
                                     ADDINFOPATH=self.additional_info_folderpath)
        self.data = import_pkl.combined_data()

    def call_dashboard(self):
        DashboardPlotter(data=self.data["data_periods"],print_settings=self.print_settings).run()

    def run(self):
        self.import_data()
        self.call_dashboard() 

# class bilateral_trade_dashboard:
#     def __init__(self,scenario_folder_path:Path,
#                  num_files_to_read:int=10):
#         self.num_files_to_read = num_files_to_read
#         self.scenario_folder_path = scenario_folder_path

#     def import_data(self):
#         import warnings
#         warnings.simplefilter(action='ignore', category=FutureWarning)
#         import_pkl = import_pkl_data(num_files_to_read=self.num_files_to_read,
#                                      SCENARIOPATH=self.scenario_folder_path)
#         self.data = import_pkl.combined_data()

#     def call_dashboard(self):
#         BT_DashboardPlotter(data=self.data["data_periods"]).run()

#     def run(self):
#         self.import_data()
#         self.call_dashboard() 

class validation_dashboard:
    def __init__(self,
                 scenario_folder_path:Path,
                 additional_info_folderpath:Path,
                 num_files_to_read:int=10,
                 only_baseline_sc:bool=True):
        self.num_files_to_read = num_files_to_read
        self.scenario_folder_path = scenario_folder_path
        self.additional_info_folderpath = additional_info_folderpath
        self.only_baseline_sc = only_baseline_sc

    def import_data(self):
        import warnings
        warnings.simplefilter(action='ignore', category=FutureWarning)
        from Toolbox.classes.import_data import import_pkl_data, import_formip_data
        import_pkl = import_pkl_data(num_files_to_read=self.num_files_to_read,
                                     ADDINFOPATH=self.additional_info_folderpath,
                                     SCENARIOPATH=self.scenario_folder_path)
        self.data = import_pkl.combined_data()

        import_formip_data = import_formip_data(timba_data=self.data, 
                                                only_baseline_sc=self.only_baseline_sc,
                                                ADDINFOPATH=self.additional_info_folderpath)
        self.formip_data = import_formip_data.load_formip_data()


    def call_dashboard(self):
        Vali_DashboardPlotter(data=self.formip_data).run()

    def run(self):
        self.import_data()
        self.call_dashboard() 

if __name__ == "__main__":
    td = timba_dashboard(num_files_to_read=4,
                         scenario_folder_path=toolbox_paths.SCINPUTPATH,
                         additional_info_folderpath= toolbox_paths.AIINPUTPATH,
                         print_settings=False)
    td.run()

    # vd = validation_dashboard(num_files_to_read=5,
    #                           scenario_folder_path=toolbox_paths.SCINPUTPATH,
    #                           additional_info_folderpath=toolbox_paths.AIINPUTPATH)
    # vd.run()